from functools import wraps

from flask_jwt_extended import get_jwt
from flask import request
import inspect


def is_admin():
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            claims = get_jwt()
            if not claims["is_admin"]:
                return ErrorResponse.forge(message=gettext('admin_privilege_required'),
                                           status_code=status.HTTP_401_UNAUTHORIZED)
            return func(*args, **kwargs)

        return wrapper

    return decorator


def validate_headers():
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            # region language
            provided_locale = request.headers.get('Accept-Language')
            if not provided_locale:
                return ErrorResponse.forge(message=gettext('missing_accept_language'))

            final_locale: str = ''
            locale_array: list = provided_locale.split("-")
            provided_language: str = 'en'
            if len(locale_array) > 0:
                provided_language = locale_array[0]

            lang_compatibility_ratio: float = 0  # 0 none, 0.5 partial, 1 full
            available: bool = next((True for locale in localization_helper.accepted_locales if provided_locale == locale['locale']), False)
            if available:
                lang_compatibility_ratio = 1.0
                final_locale = provided_locale
            else:
                available: bool = next((True for locale in localization_helper.accepted_locales if provided_language in locale['language']), False)
                if available:
                    lang_compatibility_ratio = 0.5
                else:
                    lang_compatibility_ratio = 0

            if lang_compatibility_ratio == 0:
                final_locale = constants.default_locale
            elif lang_compatibility_ratio == 0.5:
                if 'en' in provided_language:
                    final_locale = 'en-US'
                elif 'ar' in provided_language:
                    final_locale = 'ar-LB'
            # return ErrorResponse.forge(message=gettext('invalid_accept_language'))
            # endregion

            # region client
            client_os = request.headers.get('Client-OS')
            if not client_os:
                return ErrorResponse.forge(message=gettext('missing_client_os'))

            client_os_version = request.headers.get('Client-OS-Version')
            if not client_os_version:
                return ErrorResponse.forge(message=gettext('missing_client_os_version'))

            client_timezone = request.headers.get('Client-Timezone')
            if not client_timezone:
                return ErrorResponse.forge(message=gettext('missing_client_timezone'))

            client_model = request.headers.get('Client-Model')
            if not client_model:
                return ErrorResponse.forge(message=gettext('missing_client_model'))
            # endregion

            # region app
            app_version = request.headers.get('App-Version')
            if not app_version:
                return ErrorResponse.forge(message=gettext('missing_app_version'))
            # endregion

            headers: HeadersModel = HeadersModel(locale=final_locale,
                                                 client_os=client_os,
                                                 client_os_version=client_os_version,
                                                 client_timezone=client_timezone.replace(' ', ''),
                                                 client_model=client_model,
                                                 app_version=app_version)

            # check if function has argument "header"
            full_arg = inspect.getfullargspec(func)  # sig = inspect.signature(func)

            if 'headers' in full_arg.args:
                return func(*args, **kwargs, headers=headers)

            return func(*args, **kwargs)

        return wrapper

    return decorator

# class FlaskDecorators:
#     @staticmethod




# def superuser(f):
#     @wraps(f)
#     def decorated_function(*args, **kwargs):
#
#         if not g.user.superuser:
#             flash("You do not have permission to view that page", "warning")
#             abort(404)
#
#         return f(*args, **kwargs)
#
#     return decorated_function



def json_has_key(json: dict, key: str) -> bool:
    if key in json:
        return True
    return False


def json_key_value_is(json: dict, key: str, value: str) -> bool:
    if json_has_key(json=json, key=key):
        v = json[key]
        if value == v:
            return True
        else:
            return False
    else:
        raise Exception(f"provided json does not have key: {key}")


def json_key_value_contains(json: dict, key: str, content: str, sensitive: bool = None) -> bool:
    if json_has_key(json=json, key=key):
        v = json[key]
        v = v if sensitive else v.lower()
        content = content if sensitive else content.lower()
        if content in v:
            return True
        else:
            return False
    else:
        raise Exception(f"provided json does not have key: {key}")



import phonenumbers


def is_number_format_valid(number: str) -> (bool, str):
    try:
        parsed_number = phonenumbers.parse(number, None)
        return True, parsed_number
    except:
        return False, number


def is_valid_number(number: str):
    return phonenumbers.is_valid_number(number)



from abc import ABCMeta, abstractmethod
from enum import Enum
from typing import Union


class ErrorResponseInterface:
    __metaclass__ = ABCMeta

    @classmethod
    def forge_dict(cls, data: dict = None, error_code: Union[int, Enum] = None, status_code: int = None): raise NotImplementedError

    @classmethod
    def forge(cls, title: str = None, message: str = None, message_code: str = None, error_code: Union[int, Enum] = None, data: dict = None, status_code: int = None): raise NotImplementedError

    @classmethod
    def forge_localized(cls, title_key: str = None, message_key: str = None, error_code: Union[int, Enum] = None, status_code: int = None): raise NotImplementedError
    # @abstractmethod
    # def make_response(cls): return "1.0"


class SuccessResponseInterface:
    __metaclass__ = ABCMeta

    @classmethod
    def forge_dict(cls, data: dict = None, status_code: int = None): raise NotImplementedError

    @classmethod
    def forge_raw(cls, status_code: int = None, **kwargs): raise NotImplementedError

    @classmethod
    def forge(cls, title: str = None, message: str = None, data: dict = None, status_code: int = None): raise NotImplementedError


class VerboseResponseInterface:
    __metaclass__ = ABCMeta

    @classmethod
    def forge(cls, data: dict = None, status_code: int = None): raise NotImplementedError
    # @abstractmethod
    # def make_response(cls): return "1.0"



from aipkgs_requests import status
from flask import jsonify, make_response
from flask_jwt_extended import JWTManager, create_access_token, create_refresh_token, get_jwt_identity, get_jwt

from aipkgs_core.jwt.model import JWTModel


class JWTHelper:
    @staticmethod
    def fire(jwt: JWTManager):
        # The following callbacks are used for customizing jwt response/error messages.
        # The original ones may not be in a very pretty format (opinionated)
        @jwt.expired_token_loader
        def expired_token_callback(jwt_header, jwt_data):
            model = JWTModel(jwt_data)
            if model.is_access():
                return make_response(jsonify({"message": "The token has expired.", "error": "token_expired"}), status.HTTP_401_UNAUTHORIZED)
            else:
                return make_response(jsonify({"message": "The token has expired.", "error": "token_expired"}), status.HTTP_401_UNAUTHORIZED)

        @jwt.invalid_token_loader
        def invalid_token_callback(error):  # we have to keep the argument here, since it's passed in by the caller internally
            return make_response(jsonify({"message": "Signature verification failed.", "error": "invalid_token"}), status.HTTP_401_UNAUTHORIZED)

        @jwt.unauthorized_loader
        def missing_token_callback(error):
            return make_response(jsonify({
                "description": "Request does not contain an access token.",
                "error": "authorization_required",
            }), status.HTTP_401_UNAUTHORIZED)

        @jwt.needs_fresh_token_loader
        def token_not_fresh_callback(jwt_header, jwt_data):
            return make_response(jsonify({"description": "The token is not fresh.", "error": "fresh_token_required"}), status.HTTP_401_UNAUTHORIZED)

        @jwt.revoked_token_loader
        def revoked_token_callback(jwt_header, jwt_data):  # called on blacklisted
            return make_response(jsonify({"description": "The token has been revoked.", "error": "token_revoked"}), status.HTTP_401_UNAUTHORIZED)  # tokens


class AuthHelper:
    @staticmethod
    def generate_new_tokens(identity: str, fresh: bool = None, additional_claims: {} = None) -> (str, str):
        additional_claims = additional_claims if additional_claims else {}
        access_token = AuthHelper.generate_access_token(identity=identity, fresh=fresh or False, additional_claims=additional_claims)
        refresh_token = AuthHelper.generate_refresh_token(identity=identity, additional_claims=additional_claims)

        return access_token, refresh_token

    @staticmethod
    def generate_access_token(identity: str, fresh: bool = None, additional_claims: dict = None, additional_headers: dict = None) -> str:
        access_token = create_access_token(identity=identity, fresh=fresh or False, additional_claims=additional_claims, additional_headers=additional_headers)
        return access_token

    @staticmethod
    def generate_refresh_token(identity: str, additional_claims: dict = None) -> str:
        refresh_token = create_refresh_token(identity=identity, additional_claims=additional_claims)
        return refresh_token



from db import db
from models.root.root_model import RootModel


class HeadersModel:
    def __init__(self, locale: str = None, client_os: str = None, client_os_version: str = None, client_timezone: str = None, client_model: str = None, app_version: str = None):
        self.locale = locale
        self.client_os = client_os
        self.client_os_version = client_os_version
        self.client_timezone = client_timezone
        self.client_model = client_model
        self.app_version = app_version



import aipkgs_core.utils.file_helpers as ai_file_helpers
import aipkgs_core.utils.dir_helpers as ai_dir_helpers
import aipkgs_core.utils.email as ai_email
import aipkgs_core.utils.phone_number as ai_phone_number
import aipkgs_core.utils.string_helpers as ai_string_helpers
import aipkgs_core.utils.utils as ai_utils
import aipkgs_core.encryption.encryption as ai_encrypt
import aipkgs_core.utils.json as ai_json
import aipkgs_core.utils.date_helpers as ai_date_helpers
from aipkgs_core.utils.singleton import Singleton



import inspect
import os


def print_with_filename(message):
    frame_info = inspect.stack()[1]
    filename = os.path.splitext(os.path.basename(frame_info.filename))[0]
    directory = os.path.basename(os.path.dirname(frame_info.filename))
    path = f"/{directory}/{filename}"
    line_number = frame_info.lineno
    function = frame_info.function
    print(f"{path} {function}:{line_number}: {message}")
